#include <QtGui>
#include <stdlib.h>
#include <string.h>
#include <iostream>
#include <sys/time.h>
#include "clientdialog.h"
using namespace std;
ClientDialog::ClientDialog()
{
  isConnected = false;
  socket = NULL;
  createServerGroupBox();
  createAnswerGroupBox();
  createQuestionGroupBox();

  closeButton = new QPushButton("Close");
  connect(closeButton, SIGNAL(clicked()), this, SLOT(closeSocket()));

  QVBoxLayout *mainLayout = new QVBoxLayout;
  mainLayout->addWidget(serverGroupBox);
  mainLayout->addWidget(answerGroupBox);
  mainLayout->addWidget(questionGroupBox);
  mainLayout->addWidget(closeButton);
  setLayout(mainLayout);
  setWindowTitle(tr("Math Client"));
}

ClientDialog:: ~ClientDialog()
{
  delete serverGroupBox;
  delete answerGroupBox;
  delete questionGroupBox;
  delete closeButton;
}

void ClientDialog::createServerGroupBox()
{
  serverGroupBox = new QGroupBox(tr("Server Information"));
  QGridLayout *layout = new QGridLayout;
  serverLabel = new QLabel("Server: ");
  serverName = new QLineEdit("192.168.1.100");
  layout -> addWidget(serverLabel, 0, 0);
  layout -> addWidget(serverName, 0, 1);
  portLabel = new QLabel("Port: ");
  portNumber = new QLineEdit();
  connectButton = new QPushButton("Connect");
  connect(connectButton, SIGNAL(clicked()), this, SLOT(connectServer()));
  layout -> addWidget(portLabel, 1, 0);
  layout -> addWidget(portNumber, 1, 1);
  layout -> addWidget(connectButton, 2, 1);
  serverGroupBox->setLayout(layout);
}

void ClientDialog::createAnswerGroupBox()
{
  answerGroupBox = new QGroupBox(tr("Answers From Server"));
  QHBoxLayout *layout = new QHBoxLayout;
  mathAnswer = new QTextEdit;
  layout -> addWidget(mathAnswer);
  answerGroupBox -> setLayout(layout);
}

void ClientDialog::createQuestionGroupBox()
{
  questionGroupBox = new QGroupBox(tr("Question Type"));
  QHBoxLayout *layout = new QHBoxLayout;
  additionRadioButton = new QRadioButton("addition");
  subtractionRadioButton = new QRadioButton("subtraction");
  multiplicationRadioButton = new QRadioButton("multiplication");
  layout -> addWidget(additionRadioButton);
  layout -> addWidget(subtractionRadioButton);
  layout -> addWidget(multiplicationRadioButton);
  questionGroupBox -> setLayout(layout);
  connect(additionRadioButton, SIGNAL(toggled(bool)),
	  this, SLOT(changeQuestion(bool)));
  connect(subtractionRadioButton, SIGNAL(toggled(bool)),
	  this, SLOT(changeQuestion(bool)));
  connect(multiplicationRadioButton, SIGNAL(toggled(bool)),
	  this, SLOT(changeQuestion(bool)));
}
void ClientDialog::connectServer()
{
  QString hostport(serverName -> text());
  hostport.append(":");
  hostport.append(portNumber -> text());
  QString textContent(hostport);
  mathAnswer -> append(hostport);
  socket = new QTcpSocket();
  connect( socket, SIGNAL( hostFound() ),
	   this,   SLOT( reportHostFound() ) );
  connect(socket, SIGNAL(connected()), 
	  this, SLOT(reportConnected()));
  connect( socket, SIGNAL( readyRead() ),
	   this,   SLOT( getMessage() ) );
  connect( socket, SIGNAL( connectionClosed() ),
	   this,   SLOT( connectionClosed() ) );
  socket -> connectToHost(serverName -> text(), 
			  (portNumber -> text()).toInt());
}

void ClientDialog::reportConnected()
{
  mathAnswer -> append("connected");
  isConnected = true;
}

void ClientDialog::reportHostFound()
{
  mathAnswer -> append("host found");
}

void ClientDialog::closeSocket()
{
  if (isConnected == true)
    {
      socket -> write("bye\n");
      socket -> flush();
    }
  accept(); // close the dialog window
}

void ClientDialog::getMessage()
{
  int numBytes = socket->bytesAvailable();
  char data[numBytes + 1];
  socket->read( data, numBytes );
  data[numBytes] = '\0'; // terminate the char *
  QString newMessage(data);
  mathAnswer -> append(newMessage);
}

void ClientDialog::connectionClosed()
{
  mathAnswer -> append("connection closed\n");
}

void ClientDialog::changeQuestion(bool checked)
{
  QTime curTime = QTime::currentTime();
  int op1 = curTime.msec() + 1; // make it non-zero
  if (checked == false) { return; }
  QuestionType qtype;
  if (additionRadioButton -> isChecked()) 
    { qtype = ADDITION; }
  else if (subtractionRadioButton -> isChecked()) 
    { qtype = SUBTRACTION; }
  else if (multiplicationRadioButton -> isChecked()) 
    { qtype = MULTIPLICATION; }
  else
    { qtype = UNKNOWN; return; }
  int op2 = (curTime.msec() * curTime.msec()) % 1000 + 1; 
  QString question;
  question.append(QString::number(op1));
  switch (qtype)
    {
    case ADDITION:
      question.append(" + ");
      break;
    case SUBTRACTION:
      question.append(" - ");
      break;
    case MULTIPLICATION:
      question.append(" * ");
      break;
    case UNKNOWN: // do nothing for now
      break;
    }
  question.append(QString::number(op2));
  question.append("\n"); // the server expect a new line
  const char * cmsg = (question.toAscii()).data();
  socket -> write(cmsg);
  socket -> flush();
}
